package com.mortals.xhx.module.record.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.CertificateStatus;
import com.mortals.xhx.common.code.GenerateStatus;
import com.mortals.xhx.common.code.PrintStatus;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.ParamKey;
import com.mortals.xhx.module.certificate.model.CertificateCatalogEntity;
import com.mortals.xhx.module.certificate.model.CertificateCatalogTemplateEntity;
import com.mortals.xhx.module.certificate.model.CertificateClassifyQuery;
import com.mortals.xhx.module.certificate.service.CertificateCatalogService;
import com.mortals.xhx.module.certificate.service.CertificateCatalogTemplateService;
import com.mortals.xhx.module.certificate.service.CertificateClassifyService;
import com.mortals.xhx.module.print.model.PrintCatalogEntity;
import com.mortals.xhx.module.print.model.PrintCatalogQuery;
import com.mortals.xhx.module.print.model.PrintServiceEntity;
import com.mortals.xhx.module.print.model.PrintServiceQuery;
import com.mortals.xhx.module.print.service.PrintCatalogService;
import com.mortals.xhx.module.print.service.PrintServiceService;
import com.mortals.xhx.module.record.dao.PrintLogDao;
import com.mortals.xhx.module.record.dao.PrintWaitQueueDao;
import com.mortals.xhx.module.record.dao.RetainLogDao;
import com.mortals.xhx.module.record.model.*;
import com.mortals.xhx.module.record.service.PrintWaitQueueService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
* PrintWaitQueueService
* 证照打印记录 service实现
*
* @author zxfei
* @date 2022-10-22
*/
@Service("printWaitQueueService")
public class PrintWaitQueueServiceImpl extends AbstractCRUDServiceImpl<PrintWaitQueueDao, PrintWaitQueueEntity, Long> implements PrintWaitQueueService {

    /** 系统ip端口 */
    @Value("${system.ip:http://192.168.0.98:11075/}")
    private String ip;
    /** 打印api接口地址 */
    @Value("${print.aip:http://192.168.0.58:8037/api/print/GetHardWareInfo?GetType=78}")
    private String printApi;
    /** 打印机设备ip */
    @Value("${system.print:192.168.0.11}")
    private String printIp;

    @Autowired
    private PrintLogDao printLogDao;
    @Autowired
    private RetainLogDao retainLogDao;
    @Autowired
    private CertificateCatalogService certificateCatalogService;
    @Autowired
    private CertificateClassifyService certificateClassifyService;
    @Autowired
    private ParamService paramService;
    @Autowired
    private PrintServiceService printServiceService;
    @Autowired
    private PrintCatalogService printCatalogService;
    @Autowired
    private CertificateCatalogTemplateService certificateCatalogTemplateService;

    @Override
    protected void findAfter(PrintWaitQueueEntity entity, Context context, List<PrintWaitQueueEntity> list) throws AppException {
        //TODO 当前版本没有设置多个打印服务，默认只存在一个打印服务
        List<PrintServiceEntity> printServiceEntities = printServiceService.find(new PrintServiceQuery());
        PrintServiceEntity printServiceEntity = null;
        if(CollectionUtils.isNotEmpty(printServiceEntities)){
            printServiceEntity = printServiceEntities.get(0);
        }
        for(PrintWaitQueueEntity printWaitQueueEntity:list){
            if(printServiceEntity!=null){
                PrintCatalogEntity printCatalogEntity = printCatalogService.selectOne(new PrintCatalogQuery().catalogId(printWaitQueueEntity.getCatalogId()).serviceId(printServiceEntity.getId()));
                if(printCatalogEntity!=null){
                    printWaitQueueEntity.setPaperSource(printCatalogEntity.getPaperSource());
                    printWaitQueueEntity.setPrinterIp(printCatalogEntity.getPrinterIp());
                    printWaitQueueEntity.setPrinterName(printCatalogEntity.getPrinterName());
                }else {
                    //CertificateCatalogEntity catalogEntity = certificateCatalogService.get(printWaitQueueEntity.getCatalogId());
                    CertificateCatalogTemplateEntity templateEntity = certificateCatalogTemplateService.get(printWaitQueueEntity.getTemplateId());
                    printWaitQueueEntity.setPaperSource(templateEntity.getPaperSource());
                    printWaitQueueEntity.setPrinterIp(templateEntity.getPrinterIp());
                    printWaitQueueEntity.setPrinterName(templateEntity.getPrinterName());
                }
                printWaitQueueEntity.setPrinterServiceUrl(printServiceEntity.getServiceUrl());
            }else {
                //CertificateCatalogEntity catalogEntity = certificateCatalogService.get(printWaitQueueEntity.getCatalogId());
                CertificateCatalogTemplateEntity templateEntity = certificateCatalogTemplateService.get(printWaitQueueEntity.getTemplateId());
                printWaitQueueEntity.setPaperSource(templateEntity.getPaperSource());
                printWaitQueueEntity.setPrinterIp(templateEntity.getPrinterIp());
                printWaitQueueEntity.setPrinterName(templateEntity.getPrinterName());
                printWaitQueueEntity.setPrinterServiceUrl("http://127.0.0.1:8037");
            }
        }
    }

    @Override
    public void creatWaitQueueByApply(ApplyLogEntity applyLogEntity, int generateType, Context context) throws AppException {
        //CertificateCatalogEntity catalogEntity = certificateCatalogService.get(applyLogEntity.getCatalogId());
        if(generateType== GenerateStatus.ALL.getValue()){
            PrintWaitQueueEntity original = creatWaitQueue(applyLogEntity,GenerateStatus.ORIGINAL.getValue());
            dao.insert(original);
            PrintWaitQueueEntity duplicate = creatWaitQueue(applyLogEntity,GenerateStatus.DUPLICATE.getValue());
            dao.insert(duplicate);
        }else {
            PrintWaitQueueEntity printWaitQueueEntity = creatWaitQueue(applyLogEntity,generateType);
            dao.insert(printWaitQueueEntity);
        }
    }

    private PrintWaitQueueEntity creatWaitQueue(ApplyLogEntity applyLogEntity,int generateType){
        PrintWaitQueueEntity waitQueueEntity = new PrintWaitQueueEntity();
        waitQueueEntity.setApplyId(applyLogEntity.getId());
        waitQueueEntity.setRecordId(applyLogEntity.getRecordId());
        waitQueueEntity.setCatalogId(applyLogEntity.getCatalogId());
        waitQueueEntity.setCatalogCode(applyLogEntity.getCatalogCode());
        waitQueueEntity.setCatalogName(applyLogEntity.getCatalogName());
        waitQueueEntity.setTemplateId(applyLogEntity.getTemplateId());
        waitQueueEntity.setTemplateName(applyLogEntity.getTemplateName());
        waitQueueEntity.setCertificateName(applyLogEntity.getCertificateName());
        waitQueueEntity.setCertificateCode(applyLogEntity.getCertificateCode());
        waitQueueEntity.setEnterpriseName(applyLogEntity.getEnterpriseName());
        waitQueueEntity.setHolderName(applyLogEntity.getHolderName());
        waitQueueEntity.setHolderIDCardNo(applyLogEntity.getHolderIDCardNo());
        waitQueueEntity.setPickerName(applyLogEntity.getPickerName());
        waitQueueEntity.setPickerIDCardNo(applyLogEntity.getPickerIDCardNo());
        waitQueueEntity.setPreviewUrl(applyLogEntity.getPreviewUrl());
        waitQueueEntity.setCertificateUrl(applyLogEntity.getCertificateUrl());
        waitQueueEntity.setPrintStatus(YesNoEnum.NO.getValue());
        waitQueueEntity.setTotal(paramService.getParamIntValue(ParamKey.ALLOW_PRINT_MAX_COUNT));
        waitQueueEntity.setCreateUserId(applyLogEntity.getCreateUserId());
        waitQueueEntity.setCreateTime(new Date());
        return waitQueueEntity;
    }

    @Override
    public void doPrint(Long id,int printStatus,String statusRemark) {
        PrintWaitQueueEntity waitQueueEntity = this.dao.get(id);
        if(waitQueueEntity==null){
            throw new AppException("数据不存在或已修改");
        }
        PrintWaitQueueQuery data = new PrintWaitQueueQuery();
        data.setId(id);
        if(printStatus==PrintStatus.SUCCESS.getValue()){
            int total = waitQueueEntity.getTotal();
            total--;
            doPrintSuccess(waitQueueEntity);
            if(total==0){
                data.setPrintStatus(YesNoEnum.YES.getValue());
            }
            data.setTotalIncrement(-1);
        }else {
            doPrintFail(waitQueueEntity,statusRemark);
            //data.setPrintStatus(-1);
        }
        data.setUpdateTime(new Date());
        dao.update(data);

    }

    private void doPrintFail(PrintWaitQueueEntity waitQueueEntity,String statusRemark){
        Date now = new Date();
        PrintLogEntity printLogEntity = new PrintLogEntity();
        printLogEntity.setSiteId(waitQueueEntity.getSiteId());
        printLogEntity.setCatalogId(waitQueueEntity.getCatalogId());
        printLogEntity.setCatalogCode(waitQueueEntity.getCatalogCode());
        printLogEntity.setCatalogName(waitQueueEntity.getCatalogName());
        printLogEntity.setTemplateId(waitQueueEntity.getTemplateId());
        printLogEntity.setTemplateName(waitQueueEntity.getTemplateName());
        printLogEntity.setCertificateName(waitQueueEntity.getCertificateName());
        printLogEntity.setCertificateCode(waitQueueEntity.getCertificateCode());
        printLogEntity.setEnterpriseName(waitQueueEntity.getEnterpriseName());
        printLogEntity.setHolderName(waitQueueEntity.getHolderName());
        printLogEntity.setHolderIDCardNo(waitQueueEntity.getHolderIDCardNo());
        printLogEntity.setPickerName(waitQueueEntity.getPickerName());
        printLogEntity.setPickerIDCardNo(waitQueueEntity.getPickerIDCardNo());

        printLogEntity.setPrintDate(now);
        printLogEntity.setCreateUserId(waitQueueEntity.getCreateUserId());
        printLogEntity.setCreateTime(now);
        printLogEntity.setPrintStatus(PrintStatus.FAIL.getValue());
        printLogEntity.setStatusRemark(statusRemark);
        printLogDao.insert(printLogEntity);
    }

    private void doPrintSuccess(PrintWaitQueueEntity waitQueueEntity){
        RetainLogEntity retainLogEntity = new RetainLogEntity();
        Date now = new Date();
        retainLogEntity.setSiteId(waitQueueEntity.getSiteId());
        retainLogEntity.setCatalogId(waitQueueEntity.getCatalogId());
        retainLogEntity.setCatalogCode(waitQueueEntity.getCatalogCode());
        retainLogEntity.setCatalogName(waitQueueEntity.getCatalogName());
        retainLogEntity.setTemplateId(waitQueueEntity.getTemplateId());
        retainLogEntity.setTemplateName(waitQueueEntity.getTemplateName());
        retainLogEntity.setCertificateName(waitQueueEntity.getCertificateName());
        retainLogEntity.setCertificateCode(waitQueueEntity.getCertificateCode());
        retainLogEntity.setEnterpriseName(waitQueueEntity.getEnterpriseName());
        retainLogEntity.setHolderName(waitQueueEntity.getHolderName());
        retainLogEntity.setHolderIDCardNo(waitQueueEntity.getHolderIDCardNo());
        retainLogEntity.setCertificateStatus(CertificateStatus.NORMAL.getValue());
        retainLogEntity.setCreateUserId(waitQueueEntity.getCreateUserId());
        retainLogEntity.setCreateTime(now);

        PrintLogEntity printLogEntity = new PrintLogEntity();
        printLogEntity.setSiteId(waitQueueEntity.getSiteId());
        printLogEntity.setCatalogId(waitQueueEntity.getCatalogId());
        printLogEntity.setCatalogCode(waitQueueEntity.getCatalogCode());
        printLogEntity.setCatalogName(waitQueueEntity.getCatalogName());
        printLogEntity.setTemplateId(waitQueueEntity.getTemplateId());
        printLogEntity.setTemplateName(waitQueueEntity.getTemplateName());
        printLogEntity.setCertificateName(waitQueueEntity.getCertificateName());
        printLogEntity.setCertificateCode(waitQueueEntity.getCertificateCode());
        printLogEntity.setEnterpriseName(waitQueueEntity.getEnterpriseName());
        printLogEntity.setHolderName(waitQueueEntity.getHolderName());
        printLogEntity.setHolderIDCardNo(waitQueueEntity.getHolderIDCardNo());
        printLogEntity.setPickerName(waitQueueEntity.getPickerName());
        printLogEntity.setPickerIDCardNo(waitQueueEntity.getPickerIDCardNo());

        printLogEntity.setPrintDate(now);
        printLogEntity.setCreateUserId(waitQueueEntity.getCreateUserId());
        printLogEntity.setCreateTime(now);
        printLogEntity.setPrintStatus(PrintStatus.SUCCESS.getValue());
        retainLogDao.insert(retainLogEntity);
        printLogDao.insert(printLogEntity);
        CertificateCatalogEntity catalogEntity = certificateCatalogService.get(waitQueueEntity.getCatalogId());
        CertificateClassifyQuery query = new CertificateClassifyQuery();
        query.setId(catalogEntity.getClassifyId());
        query.setTotalIncrement(1);
        query.setUpdateTime(now);
        certificateClassifyService.update(query);


    }
}